@@ -7,9 +7,9 @@
 """defines ANSI SQL operations.  Contains default implementations for the abstract objects 
 in the sql module."""
 
-from sqlalchemy import schema, sql, engine, util
+from sqlalchemy import schema, sql, engine, util, sql_util
 import sqlalchemy.engine.default as default
-import string, re, sets
+import string, re, sets, weakref
 
 ANSI_FUNCS = sets.ImmutableSet([
 'CURRENT_TIME',
@@ -23,11 +23,18 @@ ANSI_FUNCS = sets.ImmutableSet([
 ])
 
 
-def create_engine():
-    return engine.ComposedSQLEngine(None, ANSIDialect())
-    
+RESERVED_WORDS = util.Set(['all', 'analyse', 'analyze', 'and', 'any', 'array', 'as', 'asc', 'asymmetric', 'authorization', 'between', 'binary', 'both', 'case', 'cast', 'check', 'collate', 'column', 'constraint', 'create', 'cross', 'current_date', 'current_role', 'current_time', 'current_timestamp', 'current_user', 'default', 'deferrable', 'desc', 'distinct', 'do', 'else', 'end', 'except', 'false', 'for', 'foreign', 'freeze', 'from', 'full', 'grant', 'group', 'having', 'ilike', 'in', 'initially', 'inner', 'intersect', 'into', 'is', 'isnull', 'join', 'leading', 'left', 'like', 'limit', 'localtime', 'localtimestamp', 'natural', 'new', 'not', 'notnull', 'null', 'off', 'offset', 'old', 'on', 'only', 'or', 'order', 'outer', 'overlaps', 'placing', 'primary', 'references', 'right', 'select', 'session_user', 'similar', 'some', 'symmetric', 'table', 'then', 'to', 'trailing', 'true', 'union', 'unique', 'user', 'using', 'verbose', 'when', 'where'])
+
+LEGAL_CHARACTERS = util.Set(string.ascii_lowercase + string.ascii_uppercase + string.digits + '_$')
+ILLEGAL_INITIAL_CHARACTERS = util.Set(string.digits + '$')
+
 class ANSIDialect(default.DefaultDialect):
-    def connect_args(self):
+    def __init__(self, cache_identifiers=True, **kwargs):
+        super(ANSIDialect,self).__init__(**kwargs)
+        self.identifier_preparer = self.preparer()
+        self.cache_identifiers = cache_identifiers
+        
+    def create_connect_args(self):
         return ([],{})
 
     def dbapi(self):
@@ -46,7 +53,7 @@ class ANSIDialect(default.DefaultDialect):
         """return an IdenfifierPreparer.
         
         This object is used to format table and column names including proper quoting and case conventions."""
-        return ANSIIdentifierPreparer()
+        return ANSIIdentifierPreparer(self)
 
 class ANSICompiler(sql.Compiled):
     """default implementation of Compiled, which compiles ClauseElements into ANSI-compliant SQL strings."""
@@ -74,7 +81,8 @@ class ANSICompiler(sql.Compiled):
         self.bindtemplate = ":%s"
         self.paramstyle = dialect.paramstyle
         self.positional = dialect.positional
-        self.preparer = dialect.preparer()
+        self.positiontup = []
+        self.preparer = dialect.identifier_preparer
         
     def after_compile(self):
         # this re will search for params like :param
@@ -84,7 +92,6 @@ class ANSICompiler(sql.Compiled):
         if self.paramstyle=='pyformat':
             self.strings[self.statement] = re.sub(match, lambda m:'%(' + m.group(1) +')s', self.strings[self.statement])
         elif self.positional:
-            self.positiontup = []
             params = re.finditer(match, self.strings[self.statement])
             for p in params:
                 self.positiontup.append(p.group(1))
@@ -128,38 +135,19 @@ class ANSICompiler(sql.Compiled):
             bindparams = {}
         bindparams.update(params)
 
-        d = sql.ClauseParameters(self.dialect)
-        if self.positional:
-            for k in self.positiontup:
-                b = self.binds[k]
-                d.set_parameter(k, b.value, b)
-        else:
-            for b in self.binds.values():
-                d.set_parameter(b.key, b.value, b)
-            
+        d = sql.ClauseParameters(self.dialect, self.positiontup)
+        for b in self.binds.values():
+            d.set_parameter(b, b.value)
+
         for key, value in bindparams.iteritems():
             try:
                 b = self.binds[key]
             except KeyError:
                 continue
-            d.set_parameter(b.key, value, b)
+            d.set_parameter(b, value)
 
         return d
 
-    def get_named_params(self, parameters):
-        """given the results of the get_params method, returns the parameters
-        in dictionary format.  For a named paramstyle, this just returns the
-        same dictionary.  For a positional paramstyle, the given parameters are
-        assumed to be in list format and are converted back to a dictionary.
-        """
-        if self.positional:
-            p = {}
-            for i in range(0, len(self.positiontup)):
-                p[self.positiontup[i]] = parameters[i]
-            return p
-        else:
-            return parameters
-    
     def default_from(self):
         """called when a SELECT statement has no froms, and no FROM clause is to be appended.  
         gives Oracle a chance to tack on a "FROM DUAL" to the string output. """
@@ -168,7 +156,7 @@ class ANSICompiler(sql.Compiled):
     def visit_label(self, label):
         if len(self.select_stack):
             self.typemap.setdefault(label.name.lower(), label.obj.type)
-        self.strings[label] = self.strings[label.obj] + " AS "  + label.name
+        self.strings[label] = self.strings[label.obj] + " AS "  + self.preparer.format_label(label)
         
     def visit_column(self, column):
         if len(self.select_stack):
@@ -183,14 +171,14 @@ class ANSICompiler(sql.Compiled):
                 if n is not None:
                     self.strings[column] = "%s.%s" % (self.preparer.format_table(column.table, use_schema=False), n)
                 elif len(column.table.primary_key) != 0:
-                    self.strings[column] = self.preparer.format_column_with_table(column.table.primary_key[0])
+                    self.strings[column] = self.preparer.format_column_with_table(list(column.table.primary_key)[0])
                 else:
                     self.strings[column] = None
             else:
                 self.strings[column] = self.preparer.format_column_with_table(column)
 
     def visit_fromclause(self, fromclause):
-        self.froms[fromclause] = fromclause.from_name
+        self.froms[fromclause] = fromclause.name
 
     def visit_index(self, index):
         self.strings[index] = index.name
@@ -248,8 +236,10 @@ class ANSICompiler(sql.Compiled):
             self.typemap.setdefault(func.name, func.type)
         if not self.apply_function_parens(func):
             self.strings[func] = ".".join(func.packagenames + [func.name])
+            self.froms[func] = self.strings[func]
         else:
             self.strings[func] = ".".join(func.packagenames + [func.name]) + "(" + string.join([self.get_str(c) for c in func.clauses], ', ') + ")"
+            self.froms[func] = self.strings[func]
         
     def visit_compound_select(self, cs):
         text = string.join([self.get_str(c) for c in cs.selects], " " + cs.keyword + " ")
@@ -259,6 +249,7 @@ class ANSICompiler(sql.Compiled):
         order_by = self.get_str(cs.order_by_clause)
         if order_by:
             text += " ORDER BY " + order_by
+        text += self.visit_select_postclauses(cs)
         if cs.parens:
             self.strings[cs] = "(" + text + ")"
         else:
@@ -298,7 +289,7 @@ class ANSICompiler(sql.Compiled):
         return self.bindtemplate % name
         
     def visit_alias(self, alias):
-        self.froms[alias] = self.get_from_text(alias.original) + " AS " + alias.name
+        self.froms[alias] = self.get_from_text(alias.original) + " AS " + self.preparer.format_alias(alias)
         self.strings[alias] = self.get_str(alias.original)
 
     def visit_select(self, select):
@@ -328,7 +319,7 @@ class ANSICompiler(sql.Compiled):
                     inner_columns[co._label] = l
                 # TODO: figure this out, a ColumnClause with a select as a parent
                 # is different from any other kind of parent
-                elif select.issubquery and isinstance(co, sql.ColumnClause) and co.table is not None and not isinstance(co.table, sql.Select):
+                elif select.issubquery and isinstance(co, sql._ColumnClause) and co.table is not None and not isinstance(co.table, sql.Select):
                     # SQLite doesnt like selecting from a subquery where the column
                     # names look like table.colname, so add a label synonomous with
                     # the column name
@@ -402,13 +393,9 @@ class ANSICompiler(sql.Compiled):
             text += " ORDER BY " + order_by
 
         text += self.visit_select_postclauses(select)
- 
-        if select.for_update:
-            text += " FOR UPDATE"
 
-        if select.nowait:
-            text += " NOWAIT"
-            
+        text += self.for_update_clause(select)
+
         if getattr(select, 'parens', False):
             self.strings[select] = "(" + text + ")"
         else:
@@ -423,13 +410,21 @@ class ANSICompiler(sql.Compiled):
         """ called when building a SELECT statement, position is after all other SELECT clauses. Most DB syntaxes put LIMIT/OFFSET here """
         return (select.limit or select.offset) and self.limit_clause(select) or ""
 
+    def for_update_clause(self, select):
+        if select.for_update:
+            return " FOR UPDATE"
+        else:
+            return ""
+
     def limit_clause(self, select):
+        text = ""
         if select.limit is not None:
-            return  " \n LIMIT " + str(select.limit)
+            text +=  " \n LIMIT " + str(select.limit)
         if select.offset is not None:
             if select.limit is None:
-                return " \n LIMIT -1"
-            return " OFFSET " + str(select.offset)
+                text += " \n LIMIT -1"
+            text += " OFFSET " + str(select.offset)
+        return text
 
     def visit_table(self, table):
         self.froms[table] = self.preparer.format_table(table)
@@ -494,7 +489,7 @@ class ANSICompiler(sql.Compiled):
         colparams = self._get_colparams(insert_stmt, default_params)
 
         def create_param(p):
-            if isinstance(p, sql.BindParamClause):
+            if isinstance(p, sql._BindParamClause):
                 self.binds[p.key] = p
                 if p.shortname is not None:
                     self.binds[p.shortname] = p
@@ -526,7 +521,7 @@ class ANSICompiler(sql.Compiled):
         self.isupdate = True
         colparams = self._get_colparams(update_stmt, default_params)
         def create_param(p):
-            if isinstance(p, sql.BindParamClause):
+            if isinstance(p, sql._BindParamClause):
                 self.binds[p.key] = p
                 self.binds[p.shortname] = p
                 return self.bindparam_string(p.key)
@@ -563,7 +558,7 @@ class ANSICompiler(sql.Compiled):
         # case one: no parameters in the statement, no parameters in the 
         # compiled params - just return binds for all the table columns
         if self.parameters is None and stmt.parameters is None:
-            return [(c, sql.bindparam(c.name, type=c.type)) for c in stmt.table.columns]
+            return [(c, sql.bindparam(c.key, type=c.type)) for c in stmt.table.columns]
 
         # if we have statement parameters - set defaults in the 
         # compiled params
@@ -582,7 +577,7 @@ class ANSICompiler(sql.Compiled):
         # now go thru compiled params, get the Column object for each key
         d = {}
         for key, value in parameters.iteritems():
-            if isinstance(key, sql.ColumnClause):
+            if isinstance(key, sql._ColumnClause):
                 d[key] = value
             else:
                 try:
@@ -596,7 +591,7 @@ class ANSICompiler(sql.Compiled):
             if d.has_key(c):
                 value = d[c]
                 if sql._is_literal(value):
-                    value = sql.bindparam(c.name, value, type=c.type)
+                    value = sql.bindparam(c.key, value, type=c.type)
                 values.append((c, value))
         return values
 
@@ -611,24 +606,46 @@ class ANSICompiler(sql.Compiled):
     def __str__(self):
         return self.get_str(self.statement)
 
-
-class ANSISchemaGenerator(engine.SchemaIterator):
-    def __init__(self, engine, proxy, connection=None, checkfirst=False, **params):
-        super(ANSISchemaGenerator, self).__init__(engine, proxy, **params)
+class ANSISchemaBase(engine.SchemaIterator):
+    def find_alterables(self, tables):
+        alterables = []
+        class FindAlterables(schema.SchemaVisitor):
+            def visit_foreign_key_constraint(self, constraint):
+                if constraint.use_alter and constraint.table in tables:
+                    alterables.append(constraint)
+        findalterables = FindAlterables()
+        for table in tables:
+            for c in table.constraints:
+                c.accept_schema_visitor(findalterables)
+        return alterables
+        
+class ANSISchemaGenerator(ANSISchemaBase):
+    def __init__(self, engine, proxy, connection, checkfirst=False, tables=None, **kwargs):
+        super(ANSISchemaGenerator, self).__init__(engine, proxy, **kwargs)
         self.checkfirst = checkfirst
+        self.tables = tables and util.Set(tables) or None
         self.connection = connection
         self.preparer = self.engine.dialect.preparer()
-    
+        self.dialect = self.engine.dialect
+        
     def get_column_specification(self, column, first_pk=False):
         raise NotImplementedError()
-        
+    
+    def visit_metadata(self, metadata):
+        collection = [t for t in metadata.table_iterator(reverse=False, tables=self.tables) if (not self.checkfirst or not self.dialect.has_table(self.connection, t.name))]
+        for table in collection:
+            table.accept_schema_visitor(self, traverse=False)
+        if self.supports_alter():
+            for alterable in self.find_alterables(collection):
+                self.add_foreignkey(alterable)
+                
     def visit_table(self, table):
-        # the single whitespace before the "(" is significant
-        # as its MySQL's method of indicating a table name and not a reserved word.
-        # feel free to localize this logic to the mysql module
-        if self.checkfirst and self.engine.dialect.has_table(self.connection, table.name):
-            return
-            
+        for column in table.columns:
+            if column.default is not None:
+                column.default.accept_schema_visitor(self, traverse=False)
+            #if column.onupdate is not None:
+            #    column.onupdate.accept_schema_visitor(visitor, traverse=False)
+        
         self.append("\nCREATE TABLE " + self.preparer.format_table(table) + " (")
         
         separator = "\n"
@@ -641,15 +658,17 @@ class ANSISchemaGenerator(engine.SchemaIterator):
             self.append("\t" + self.get_column_specification(column, first_pk=column.primary_key and not first_pk))
             if column.primary_key:
                 first_pk = True
-
+            for constraint in column.constraints:
+                constraint.accept_schema_visitor(self, traverse=False)
+                
         for constraint in table.constraints:
-            constraint.accept_schema_visitor(self)            
+            constraint.accept_schema_visitor(self, traverse=False)
 
         self.append("\n)%s\n\n" % self.post_create_table(table))
-        self.execute()        
+        self.execute()
         if hasattr(table, 'indexes'):
             for index in table.indexes:
-                self.visit_index(index)
+                index.accept_schema_visitor(self, traverse=False)
         
     def post_create_table(self, table):
         return ''
@@ -664,18 +683,40 @@ class ANSISchemaGenerator(engine.SchemaIterator):
             return None
 
     def _compile(self, tocompile, parameters):
+        """compile the given string/parameters using this SchemaGenerator's dialect."""
         compiler = self.engine.dialect.compiler(tocompile, parameters)
         compiler.compile()
         return compiler
 
+    def visit_check_constraint(self, constraint):
+        self.append(", \n\t")
+        if constraint.name is not None:
+            self.append("CONSTRAINT %s " % constraint.name)
+        self.append(" CHECK (%s)" % constraint.sqltext)
+        
     def visit_primary_key_constraint(self, constraint):
         if len(constraint) == 0:
             return
-        self.append(", \n")
-        self.append("\tPRIMARY KEY (%s)" % string.join([self.preparer.format_column(c) for c in constraint],', '))
-            
+        self.append(", \n\tPRIMARY KEY ")
+        if constraint.name is not None:
+            self.append("%s " % constraint.name)
+        self.append("(%s)" % (string.join([self.preparer.format_column(c) for c in constraint],', ')))
+    
+    def supports_alter(self):
+        return True
+                        
     def visit_foreign_key_constraint(self, constraint):
+        if constraint.use_alter and self.supports_alter():
+            return
         self.append(", \n\t ")
+        self.define_foreign_key(constraint)
+    
+    def add_foreignkey(self, constraint):
+        self.append("ALTER TABLE %s ADD " % self.preparer.format_table(constraint.table))
+        self.define_foreign_key(constraint)
+        self.execute()
+        
+    def define_foreign_key(self, constraint):
         if constraint.name is not None:
             self.append("CONSTRAINT %s " % constraint.name)
         self.append("FOREIGN KEY(%s) REFERENCES %s (%s)" % (
@@ -688,6 +729,13 @@ class ANSISchemaGenerator(engine.SchemaIterator):
         if constraint.onupdate is not None:
             self.append(" ON UPDATE %s" % constraint.onupdate)
 
+    def visit_unique_constraint(self, constraint):
+        self.append(", \n\t")
+        if constraint.name is not None:
+            self.append("CONSTRAINT %s " % constraint.name)
+        self.append(" UNIQUE ")
+        self.append("(%s)" % (string.join([self.preparer.format_column(c) for c in constraint],', ')))
+
     def visit_column(self, column):
         pass
 
@@ -700,31 +748,48 @@ class ANSISchemaGenerator(engine.SchemaIterator):
                        string.join([self.preparer.format_column(c) for c in index.columns], ', ')))
         self.execute()
         
-class ANSISchemaDropper(engine.SchemaIterator):
-    def __init__(self, engine, proxy, connection=None, checkfirst=False, **params):
-        super(ANSISchemaDropper, self).__init__(engine, proxy, **params)
+class ANSISchemaDropper(ANSISchemaBase):
+    def __init__(self, engine, proxy, connection, checkfirst=False, tables=None, **kwargs):
+        super(ANSISchemaDropper, self).__init__(engine, proxy, **kwargs)
         self.checkfirst = checkfirst
+        self.tables = tables
         self.connection = connection
         self.preparer = self.engine.dialect.preparer()
+        self.dialect = self.engine.dialect
+
+    def visit_metadata(self, metadata):
+        collection = [t for t in metadata.table_iterator(reverse=True, tables=self.tables) if (not self.checkfirst or  self.dialect.has_table(self.connection, t.name))]
+        if self.supports_alter():
+            for alterable in self.find_alterables(collection):
+                self.drop_foreignkey(alterable)
+        for table in collection:
+            table.accept_schema_visitor(self, traverse=False)
+
+    def supports_alter(self):
+        return True
 
     def visit_index(self, index):
         self.append("\nDROP INDEX " + index.name)
         self.execute()
+
+    def drop_foreignkey(self, constraint):
+        self.append("ALTER TABLE %s DROP CONSTRAINT %s" % (self.preparer.format_table(constraint.table), constraint.name))
+        self.execute()
         
     def visit_table(self, table):
-        # NOTE: indexes on the table will be automatically dropped, so
-        # no need to drop them individually
-        if self.checkfirst and not self.engine.dialect.has_table(self.connection, table.name):
-            return
+        for column in table.columns:
+            if column.default is not None:
+                column.default.accept_schema_visitor(self, traverse=False)
+
         self.append("\nDROP TABLE " + self.preparer.format_table(table))
         self.execute()
 
 class ANSIDefaultRunner(engine.DefaultRunner):
     pass
 
-class ANSIIdentifierPreparer(schema.SchemaVisitor):
-    """Transforms identifiers of SchemaItems into ANSI-Compliant delimited identifiers where required"""
-    def __init__(self, initial_quote='"', final_quote=None, omit_schema=False):
+class ANSIIdentifierPreparer(object):
+    """handles quoting and case-folding of identifiers based on options"""
+    def __init__(self, dialect, initial_quote='"', final_quote=None, omit_schema=False):
         """Constructs a new ANSIIdentifierPreparer object.
         
         initial_quote - Character that begins a delimited identifier
@@ -732,12 +797,11 @@ class ANSIIdentifierPreparer(schema.SchemaVisitor):
         
         omit_schema - prevent prepending schema name. useful for databases that do not support schemae
         """
+        self.dialect = dialect
         self.initial_quote = initial_quote
         self.final_quote = final_quote or self.initial_quote
         self.omit_schema = omit_schema
-        self.strings = {}
-        self.__visited = util.Set()
-        
+        self.__strings = {}
     def _escape_identifier(self, value):
         """escape an identifier.
         
@@ -759,56 +823,81 @@ class ANSIIdentifierPreparer(schema.SchemaVisitor):
         # some tests would need to be rewritten if this is done.
         #return value.upper()
     
-    def visit_table(self, table):
-        if table in self.__visited:
-            return
-        if table.quote:
-            self.strings[table] = self._quote_identifier(table.name)
-        else:
-            self.strings[table] = table.name # TODO: case folding ?
-        if table.schema:
-            if table.quote_schema:
-                self.strings[(table, 'schema')] = self._quote_identifier(table.schema)
-            else: 
-                self.strings[(table, 'schema')] = table.schema # TODO: case folding ?
-            
-    def visit_column(self, column):
-        if column in self.__visited:
-            return
-        if column.quote:
-            self.strings[column] = self._quote_identifier(column.name)
-        else:
-            self.strings[column] = column.name # TODO: case folding ?
-    
-    def __start_visit(self, obj):
-        if obj in self.__visited:
-            return
-        if isinstance(obj, schema.SchemaItem):
-            obj.accept_schema_visitor(self)
-        self.__visited.add(obj)
-         
-    def __prepare_table(self, table, use_schema=False):
-        self.__start_visit(table)
-        if not self.omit_schema and use_schema and (table, 'schema') in self.strings:
-            return self.strings[(table, 'schema')] + "." + self.strings.get(table, table.name)
-        else:
-            return self.strings.get(table, table.name)
+    def _reserved_words(self):
+        return RESERVED_WORDS
 
-    def __prepare_column(self, column, use_table=True, **kwargs):
-        self.__start_visit(column)
-        if use_table:
-            return self.__prepare_table(column.table, **kwargs) + "." + self.strings.get(column, column.name)
-        else:
-            return self.strings.get(column, column.name)
+    def _legal_characters(self):
+        return LEGAL_CHARACTERS
     
+    def _illegal_initial_characters(self):
+        return ILLEGAL_INITIAL_CHARACTERS
+        
+    def _requires_quotes(self, value, case_sensitive):
+        """return true if the given identifier requires quoting."""
+        return \
+            value in self._reserved_words() \
+            or (value[0] in self._illegal_initial_characters()) \
+            or bool(len([x for x in str(value) if x not in self._legal_characters()])) \
+            or (case_sensitive and value.lower() != value)
+    
+    def __generic_obj_format(self, obj, ident):
+        if getattr(obj, 'quote', False):
+            return self._quote_identifier(ident)
+        if self.dialect.cache_identifiers:
+            case_sens = getattr(obj, 'case_sensitive', None)
+            try:
+                return self.__strings[(ident, case_sens)]
+            except KeyError:
+                if self._requires_quotes(ident, getattr(obj, 'case_sensitive', ident == ident.lower())):
+                    self.__strings[(ident, case_sens)] = self._quote_identifier(ident)
+                else:
+                    self.__strings[(ident, case_sens)] = ident
+                return self.__strings[(ident, case_sens)]
+        else:
+            if self._requires_quotes(ident, getattr(obj, 'case_sensitive', ident == ident.lower())):
+                return self._quote_identifier(ident)
+            else:
+                return ident
+            
+    def should_quote(self, object):
+        return object.quote or self._requires_quotes(object.name, object.case_sensitive) 
+ 
+    def is_natural_case(self, object):
+        return object.quote or self._requires_quotes(object.name, object.case_sensitive)
+        
+    def format_sequence(self, sequence):
+        return self.__generic_obj_format(sequence, sequence.name)
+    
+    def format_label(self, label):
+        return self.__generic_obj_format(label, label.name)
+
+    def format_alias(self, alias):
+        return self.__generic_obj_format(alias, alias.name)
+        
     def format_table(self, table, use_schema=True):
         """Prepare a quoted table and schema name"""
-        return self.__prepare_table(table, use_schema=use_schema)
+        result = self.__generic_obj_format(table, table.name)
+        if use_schema and getattr(table, "schema", None):
+            result = self.__generic_obj_format(table, table.schema) + "." + result
+        return result
     
-    def format_column(self, column):
+    def format_column(self, column, use_table=False):
         """Prepare a quoted column name """
-        return self.__prepare_column(column, use_table=False)
-    
+        # TODO: isinstance alert !  get ColumnClause and Column to better
+        # differentiate themselves
+        if isinstance(column, schema.SchemaItem):
+            if use_table:
+                return self.format_table(column.table, use_schema=False) + "." + self.__generic_obj_format(column, column.name)
+            else:
+                return self.__generic_obj_format(column, column.name)
+        else:
+            # literal textual elements get stuck into ColumnClause alot, which shouldnt get quoted
+            if use_table:
+                return column.table.name + "." + column.name
+            else:
+                return column.name
+            
     def format_column_with_table(self, column):
         """Prepare a quoted column name with table name"""
-        return self.__prepare_column(column)
+        return self.format_column(column, use_table=True)
+
