@@ -5,7 +5,7 @@
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 __all__ = [ 'TypeEngine', 'TypeDecorator', 'NullTypeEngine',
-            'INT', 'CHAR', 'VARCHAR', 'TEXT', 'FLOAT', 'DECIMAL', 
+            'INT', 'CHAR', 'VARCHAR', 'NCHAR', 'TEXT', 'FLOAT', 'DECIMAL', 
             'TIMESTAMP', 'DATETIME', 'CLOB', 'BLOB', 'BOOLEAN', 'String', 'Integer', 'Smallinteger',
             'Numeric', 'Float', 'DateTime', 'Date', 'Time', 'Binary', 'Boolean', 'Unicode', 'PickleType', 'NULLTYPE',
         'SMALLINT', 'DATE', 'TIME'
@@ -27,6 +27,17 @@ class AbstractType(object):
             return self._impl_dict
     impl_dict = property(_get_impl_dict)
 
+    def copy_value(self, value):
+        return value
+    def compare_values(self, x, y):
+        return x is y
+    def is_mutable(self):
+        return False
+    def get_dbapi_type(self, dbapi):
+        """return the corresponding type object from the underlying DBAPI, if any.
+        
+        this can be useful for calling setinputsizes(), for example."""
+        return None
             
 class TypeEngine(AbstractType):
     def __init__(self, *args, **params):
@@ -75,6 +86,9 @@ class TypeDecorator(AbstractType):
             tt.impl = typedesc
             self.impl_dict[dialect] = tt
             return tt
+    def __getattr__(self, key):
+        """proxy all other undefined accessors to the underlying implementation."""
+        return getattr(self.impl, key)
     def get_col_spec(self):
         return self.impl.get_col_spec()
     def convert_bind_param(self, value, dialect):
@@ -85,7 +99,24 @@ class TypeDecorator(AbstractType):
         instance = self.__class__.__new__(self.__class__)
         instance.__dict__.update(self.__dict__)
         return instance
-        
+    def get_dbapi_type(self, dbapi):
+        return self.impl.get_dbapi_type(dbapi)
+    def copy_value(self, value):
+        return self.impl.copy_value(value)
+    def compare_values(self, x, y):
+        return self.impl.compare_values(x,y)
+    def is_mutable(self):
+        return self.impl.is_mutable()
+
+class MutableType(object):
+    """a mixin that marks a Type as holding a mutable object"""
+    def is_mutable(self):
+        return True
+    def copy_value(self, value):
+        raise NotImplementedError()
+    def compare_values(self, x, y):
+        return x == y
+    
 def to_instance(typeobj):
     if typeobj is None:
         return NULLTYPE
@@ -122,7 +153,6 @@ class NullTypeEngine(TypeEngine):
     def convert_result_value(self, value, dialect):
         return value
 
-    
 class String(TypeEngine):
     def __new__(cls, *args, **kwargs):
         if cls is not String or len(args) > 0 or kwargs.has_key('length'):
@@ -143,7 +173,11 @@ class String(TypeEngine):
             return value
         else:
             return value.decode(dialect.encoding)
-            
+    def get_dbapi_type(self, dbapi):
+        return dbapi.STRING
+    def compare_values(self, x, y):
+        return x == y
+        
 class Unicode(TypeDecorator):
     impl = String
     def convert_bind_param(self, value, dialect):
@@ -159,8 +193,9 @@ class Unicode(TypeDecorator):
         
 class Integer(TypeEngine):
     """integer datatype"""
-    pass
-    
+    def get_dbapi_type(self, dbapi):
+        return dbapi.NUMBER
+        
 class SmallInteger(Integer):
     """ smallint datatype """
     pass
@@ -172,6 +207,8 @@ class Numeric(TypeEngine):
         self.length = length
     def adapt(self, impltype):
         return impltype(precision=self.precision, length=self.length)
+    def get_dbapi_type(self, dbapi):
+        return dbapi.NUMBER
 
 class Float(Numeric):
     def __init__(self, precision = 10):
@@ -181,15 +218,26 @@ class Float(Numeric):
 
 class DateTime(TypeEngine):
     """implements a type for datetime.datetime() objects"""
-    pass
-
+    def __init__(self, timezone=True):
+        self.timezone = timezone
+    def adapt(self, impltype):
+        return impltype(timezone=self.timezone)
+    def get_dbapi_type(self, dbapi):
+        return dbapi.DATETIME
+        
 class Date(TypeEngine):
     """implements a type for datetime.date() objects"""
-    pass
+    def get_dbapi_type(self, dbapi):
+        return dbapi.DATETIME
 
 class Time(TypeEngine):
     """implements a type for datetime.time() objects"""
-    pass
+    def __init__(self, timezone=True):
+        self.timezone = timezone
+    def adapt(self, impltype):
+        return impltype(timezone=self.timezone)
+    def get_dbapi_type(self, dbapi):
+        return dbapi.DATETIME
 
 class Binary(TypeEngine):
     def __init__(self, length=None):
@@ -203,23 +251,38 @@ class Binary(TypeEngine):
         return value
     def adapt(self, impltype):
         return impltype(length=self.length)
+    def get_dbapi_type(self, dbapi):
+        return dbapi.BINARY
 
-class PickleType(TypeDecorator):
+class PickleType(MutableType, TypeDecorator):
     impl = Binary
-    def __init__(self, protocol=pickle.HIGHEST_PROTOCOL, pickler=None):
-       self.protocol = protocol
-       self.pickler = pickler or pickle
-       super(PickleType, self).__init__()
+    def __init__(self, protocol=pickle.HIGHEST_PROTOCOL, pickler=None, mutable=True):
+        self.protocol = protocol
+        self.pickler = pickler or pickle
+        self.mutable = mutable
+        super(PickleType, self).__init__()
     def convert_result_value(self, value, dialect):
-      if value is None:
-          return None
-      buf = self.impl.convert_result_value(value, dialect)
-      return self.pickler.loads(str(buf))
+        if value is None:
+            return None
+        buf = self.impl.convert_result_value(value, dialect)
+        return self.pickler.loads(str(buf))
     def convert_bind_param(self, value, dialect):
-      if value is None:
-          return None
-      return self.impl.convert_bind_param(self.pickler.dumps(value, self.protocol), dialect)
-
+        if value is None:
+            return None
+        return self.impl.convert_bind_param(self.pickler.dumps(value, self.protocol), dialect)
+    def copy_value(self, value):
+        if self.mutable:
+            return self.pickler.loads(self.pickler.dumps(value, self.protocol))
+        else:
+            return value
+    def compare_values(self, x, y):
+        if self.mutable:
+            return self.pickler.dumps(x, self.protocol) == self.pickler.dumps(y, self.protocol)
+        else:
+            return x is y
+    def is_mutable(self):
+        return self.mutable
+        
 class Boolean(TypeEngine):
     pass
 
@@ -236,6 +299,7 @@ class TIME(Time): pass
 class CLOB(String): pass
 class VARCHAR(String): pass
 class CHAR(String):pass
+class NCHAR(Unicode):pass
 class BLOB(Binary): pass
 class BOOLEAN(Boolean): pass
 
