@@ -3,14 +3,16 @@ import unittest
 
 def suite():
     modules_to_test = (
+	'orm.attributes',
         'orm.mapper',
-        'orm.selectresults',
         'orm.lazytest1',
         'orm.eagertest1',
         'orm.eagertest2',
+        'orm.eagertest3',
         
         'orm.sessioncontext', 
-        'orm.objectstore',
+        'orm.unitofwork',
+        'orm.session',
         'orm.cascade',
         'orm.relationships',
         'orm.association',
