@@ -1,3 +1,6 @@
+import sys
+sys.path.insert(0, './lib/')
+
 import gc
 
 import random, string
