@@ -23,7 +23,8 @@ class Logger(object):
     def write(self, msg):
         if echo:
             local_stdout.write(msg)
-sys.stdout = Logger()    
+    def flush(self):
+        pass
 
 def echo_text(text):
     print text
@@ -39,11 +40,12 @@ def parse_argv():
 
     parser = optparse.OptionParser(usage = "usage: %prog [options] files...")
     parser.add_option("--dburi", action="store", dest="dburi", help="database uri (overrides --db)")
-    parser.add_option("--db", action="store", dest="db", default="sqlite", help="prefab database uri (sqlite, sqlite_file, postgres, mysql, oracle, oracle8, mssql)")
+    parser.add_option("--db", action="store", dest="db", default="sqlite", help="prefab database uri (sqlite, sqlite_file, postgres, mysql, oracle, oracle8, mssql, firebird)")
     parser.add_option("--mockpool", action="store_true", dest="mockpool", help="use mock pool")
     parser.add_option("--verbose", action="store_true", dest="verbose", help="full debug echoing")
-    parser.add_option("--noecho", action="store_true", dest="noecho", help="Disable SQL statement echoing")
     parser.add_option("--quiet", action="store_true", dest="quiet", help="be totally quiet")
+    parser.add_option("--log-info", action="append", dest="log_info", help="turn on info logging for <LOG> (multiple OK)")
+    parser.add_option("--log-debug", action="append", dest="log_debug", help="turn on debug logging for <LOG> (multiple OK)")
     parser.add_option("--nothreadlocal", action="store_true", dest="nothreadlocal", help="dont use thread-local mod")
     parser.add_option("--enginestrategy", action="store", default=None, dest="enginestrategy", help="engine strategy (plain or threadlocal, defaults to SA default)")
 
@@ -73,9 +75,11 @@ def parse_argv():
             opts = {'use_ansi':False}
         elif DBTYPE == 'mssql':
             db_uri = 'mssql://scott:tiger@SQUAWK\\SQLEXPRESS/test'
+        elif DBTYPE == 'firebird':
+            db_uri = 'firebird://sysdba:s@localhost/tmp/test.fdb'
 
     if not db_uri:
-        raise "Could not create engine.  specify --db <sqlite|sqlite_file|postgres|mysql|oracle|oracle8|mssql> to test runner."
+        raise "Could not create engine.  specify --db <sqlite|sqlite_file|postgres|mysql|oracle|oracle8|mssql|firebird> to test runner."
 
     if not options.nothreadlocal:
         __import__('sqlalchemy.mods.threadlocal')
@@ -90,10 +94,19 @@ def parse_argv():
     if options.enginestrategy is not None:
         opts['strategy'] = options.enginestrategy    
     if options.mockpool:
-        db = engine.create_engine(db_uri, echo=(not options.noecho), default_ordering=True, poolclass=MockPool, **opts)
+        db = engine.create_engine(db_uri, default_ordering=True, poolclass=MockPool, **opts)
     else:
-        db = engine.create_engine(db_uri, echo=(not options.noecho), default_ordering=True, **opts)
+        db = engine.create_engine(db_uri, default_ordering=True, **opts)
     db = EngineAssert(db)
+
+    import logging
+    logging.basicConfig()
+    if options.log_info is not None:
+        for elem in options.log_info:
+            logging.getLogger(elem).setLevel(logging.INFO)
+    if options.log_debug is not None:
+        for elem in options.log_debug:
+            logging.getLogger(elem).setLevel(logging.DEBUG)
     metadata = sqlalchemy.BoundMetaData(db)
     
 def unsupported(*dbs):
@@ -146,18 +159,22 @@ class PersistTest(unittest.TestCase):
 class MockPool(pool.Pool):
     """this pool is hardcore about only one connection being used at a time."""
     def __init__(self, creator, **params):
-        pool.Pool.__init__(self, **params)
-        self.connection = creator()
+        pool.Pool.__init__(self, creator, **params)
+        self.connection = pool._ConnectionRecord(self)
         self._conn = self.connection
         
     def status(self):
         return "MockPool"
 
+    def create_connection(self):
+        raise "Invalid"
+
     def do_return_conn(self, conn):
         assert conn is self._conn and self.connection is None
         self.connection = conn
 
-    def do_return_invalid(self):
+    def do_return_invalid(self, conn):
+        pass
         raise "Invalid"
 
     def do_get(self):
@@ -347,10 +364,12 @@ parse_argv()
 
                     
 def runTests(suite):
+    sys.stdout = Logger()    
     runner = unittest.TextTestRunner(verbosity = quiet and 1 or 2)
     runner.run(suite)
     
 def main():
+    
     if len(sys.argv[1:]):
         suite =unittest.TestLoader().loadTestsFromNames(sys.argv[1:], __import__('__main__'))
     else:
